@@ -2,6 +2,7 @@ package Amazon::Credentials;
 
 use strict;
 use warnings;
+
 use 5.010;
 
 use parent qw/Exporter Class::Accessor::Fast/;
@@ -25,7 +26,6 @@ __PACKAGE__->mk_accessors(
     insecure
     logger
     order
-    passkey
     print_error
     profile
     raise_error
@@ -58,37 +58,44 @@ use Scalar::Util qw/reftype/;
 use List::Util qw/pairs any/;
 
 use constant { ## no critic (ValuesAndExpressions::ProhibitConstantPragma, NamingConventions::Capitalization)
-  AWS_METADATA_BASE_URL            => q{http://169.254.169.254/},
+  AWS_AVAILABILITY_ZONE_URL =>
+    q{latest/meta-data/placement/availability-zone},
   AWS_CONTAINER_CREDENTIALS_URL    => q{http://169.254.170.2/},
   AWS_IAM_SECURITY_CREDENTIALS_URL =>
     q{latest/meta-data/iam/security-credentials/},
-  AWS_AVAILABILITY_ZONE_URL =>
-    q{latest/meta-data/placement/availability-zone},
-  COMMA                   => q{,},
-  DEFAULT_CIPHER          => q{Cipher::AES},
-  DEFAULT_SEARCH_ORDER    => q{env,container,role,file},
-  DEFAULT_TIMEOUT         => 2,
-  DEFAULT_WINDOW_INTERVAL => 5,
-  EMPTY                   => q{},
-  SECONDS_IN_MINUTE       => 60,
-  SECONDS_IN_HOUR         => 3600,
-  INSECURE_MODE           => 2,
-  IMDSv2_URL              => q{latest/api/token},
-  IMDSv2_TTL_HEADER       => q{x-aws-ec2-metadata-token-ttl-seconds},
-  IMDSv2_HEADER           => q{x-aws-ec2-metadata-token},
-  IMDSv2_DEFAULT_TTL      => 21_600,
-  ISO8601_FORMAT          => q{%Y-%m-%dT%H:%M:%S%z},
-  PASSKEY_FORMAT          => q{%08X%08x},
-  RANDOM_VALUE            => 0xffffffff,
-  SLASH                   => q{/},
-  TRUE                    => 1,
-  FALSE                   => 0,
-  LOG_FORMAT              => qq{ %s [%s] %s\n },
+  AWS_METADATA_BASE_URL         => q{http://169.254.169.254/},
+  CACHE_DIR                     => '.aws/sso/cache',
+  COMMA                         => q{,},
+  DEFAULT_CIPHER                => q{Cipher::AES},
+  DEFAULT_REGION                => 'us-east-1',
+  DEFAULT_SEARCH_ORDER          => q{env,container,role,file},
+  DEFAULT_TIMEOUT               => 2,
+  DEFAULT_WINDOW_INTERVAL       => 5,
+  EMPTY                         => q{},
+  FALSE                         => 0,
+  GET_ROLE_CREDENTIALS_HOSTNAME => 'portal.sso.%s.amazonaws.com',
+  GET_ROLE_CREDENTIALS_QUERY    => '?account_id=%s&role_name=%s',
+  GET_ROLE_CREDENTIALS_URI      => 'federation/credentials',
+  IMDSv2_DEFAULT_TTL            => 21_600,
+  IMDSv2_HEADER                 => q{x-aws-ec2-metadata-token},
+  IMDSv2_TTL_HEADER             => q{x-aws-ec2-metadata-token-ttl-seconds},
+  IMDSv2_URL                    => q{latest/api/token},
+  INSECURE_MODE                 => 2,
+  ISO8601_FORMAT                => q{%Y-%m-%dT%H:%M:%S%z},
+  LOG_FORMAT                    => qq{ %s [%s] %s\n },
+  PASSKEY_FORMAT                => q{%08X%08x},
+  RANDOM_VALUE                  => 0xffffffff,
+  SECONDS_IN_HOUR               => 3600,
+  SECONDS_IN_MINUTE             => 60,
+  SLASH                         => q{/},
+  TRUE                          => 1,
+  X_AMZ_SSO_BEARER_TOKEN        => ':x-amz-sso_bearer_token',
 };
 
-our $VERSION = '1.1.9';
+our $VERSION = '1.1.10'; ## no critic (ValuesAndExpressions::RequireInterpolationOfMetachars)
 
-our @EXPORT_OK = qw{ create_passkey };
+our @EXPORT_OK
+  = qw{ create_passkey set_sso_credentials get_role_credentials };
 
 # we only log at debug level, create a default logger
 {
@@ -108,6 +115,8 @@ our @EXPORT_OK = qw{ create_passkey };
   };
 }
 
+caller or __PACKAGE__->main();
+
 ########################################################################
 sub new {
 ########################################################################
@@ -115,13 +124,15 @@ sub new {
 
   my $options = ref $args[0] ? $args[0] : {@args};
 
+  my $passkey = delete $options->{passkey};
+
   my $self = $class->SUPER::new($options);
 
   $self->_set_defaults;
 
   $self->_init_logger;
 
-  $self->_init_encryption;
+  $self->_init_encryption($passkey);
 
   if ( $self->get_insecure ) {
     $self->get_logger->debug( "!! CAUTION !!\n"
@@ -457,7 +468,7 @@ sub get_region_from_config {
     $region = $ini->{'default'}->{'region'};
   } ## end if ( $ini->{'default'})
 
-  $self->get_logger->debug( "default region: " . $region // "undef" );
+  $self->get_logger->debug( "default region: " . $region // 'undef' );
 
   return $region;
 } ## end sub get_region_from_config
@@ -874,7 +885,8 @@ sub format_credentials {
 
   my $credential_keys = $self->credential_keys;
 
-  return join q{}, map { sprintf $format, $_, $credential_keys->{$_} }
+  return join EMPTY,
+    map { sprintf $format, $_, $credential_keys->{$_} // EMPTY }
     keys %{$credential_keys};
 } ## end sub format_credentials
 
@@ -1055,6 +1067,77 @@ sub rotate_credentials {
   goto &rotate_passkey;
 } ## end sub rotate_credentials
 
+########################################################################
+sub set_sso_credentials {
+########################################################################
+  my ( $role_name, $account_id, $region ) = @_;
+
+  croak "usage: __PACKAGE__::set_sso_credentials(role-name, account-id)\n"
+    if ref $role_name;
+
+  my $credentials = get_role_credentials(
+    role_name  => $role_name,
+    account_id => $account_id,
+    region     => $region // EMPTY,
+  );
+
+  if ($credentials) {
+    foreach my $p (
+      pairs qw{
+      accessKeyId AWS_ACCESS_KEY_ID
+      secretAccessKey AWS_SECRET_ACCESS_KEY
+      sessionToken AWS_SESSION_TOKEN
+      }
+    ) {
+
+      $ENV{ $p->[1] } = $credentials->{ $p->[0] };
+    }
+
+  }
+
+  return $credentials;
+}
+
+########################################################################
+sub get_role_credentials {
+########################################################################
+  my (%args) = @_;
+
+  my ( $account_id, $role_name, $access_token, $region )
+    = @args{qw{account_id role_name access_token region}};
+
+  if ( !$access_token ) {
+    $access_token = _get_access_token();
+  }
+
+  croak 'no access token'
+    if !$access_token;
+
+  $region ||= $ENV{AWS_REGION} || $ENV{AWS_DEFAULT_REGION} || DEFAULT_REGION;
+
+  my $ua = LWP::UserAgent->new;
+  $ua->default_header( X_AMZ_SSO_BEARER_TOKEN, $access_token );
+
+  my $host = sprintf GET_ROLE_CREDENTIALS_HOSTNAME, $region;
+
+  my $query = sprintf GET_ROLE_CREDENTIALS_QUERY, $account_id, $role_name;
+
+  my $url = sprintf 'https://%s/%s%s', $host, GET_ROLE_CREDENTIALS_URI,
+    $query;
+
+  my $rsp = $ua->get($url);
+
+  if ( $rsp && !$rsp->is_success ) {
+    croak $rsp->status_line . q{:} . Dumper( $rsp->content );
+  }
+  elsif ( !$rsp ) {
+    croak "no response from get($url)\n";
+  }
+
+  return
+    eval { return JSON::PP->new->decode( $rsp->content )->{roleCredentials}; };
+}
+
 # +-----------------+
 # | PRIVATE METHODS |
 # +-----------------+
@@ -1117,6 +1200,22 @@ sub _set_defaults {
   return $self;
 } ## end sub _set_defaults
 
+{
+  my $passkey;
+
+  sub get_passkey {
+    return $passkey;
+  }
+
+  sub set_passkey {
+    my ( $self, $key ) = @_;
+
+    $passkey = $key;
+
+    return $passkey;
+  }
+}
+
 ########################################################################
 sub _fetch_passkey {
 ########################################################################
@@ -1137,7 +1236,9 @@ sub _fetch_passkey {
 ########################################################################
 sub _init_encryption {
 ########################################################################
-  my ($self) = @_;
+  my ( $self, $passkey ) = @_;
+
+  $self->set_passkey($passkey);
 
   # if one is set, both must be set
   if ( $self->get_encrypt || $self->get_decrypt ) {
@@ -1234,6 +1335,137 @@ sub _create_metadata_url {
   return AWS_METADATA_BASE_URL . $url;
 } ## end sub _create_metadata_url
 
+########################################################################
+sub _get_access_token {
+########################################################################
+  my ($home) = @_;
+
+  require File::Find;
+  File::Find->import('find');
+
+  $home //= $ENV{HOME};
+  my $cache_dir = sprintf '%s/%s', $home, CACHE_DIR;
+
+  croak "no $cache_dir found."
+    if !-d $cache_dir;
+
+  my $access_token;
+
+  eval {
+
+    find(
+      sub {
+        return if !/[.]json$/xsm;
+
+        local $RS = undef;
+
+        open my $fh, '<', $File::Find::name
+          or croak 'could not open ' . $File::Find::name;
+
+        my $json = eval { JSON::PP->new->decode(<$fh>) };
+
+        close $fh;
+
+        if ( ref $json && $json->{accessToken} ) {
+          $access_token = $json->{accessToken};
+          croak 'found';
+        }
+
+        return;
+      },
+      $cache_dir
+    );
+  };
+
+  return $access_token;
+}
+
+#######################################################################
+sub help {
+########################################################################
+
+  print {*STDOUT} <<'EOT';
+amazon-credentials.sh options
+
+Formats credentials found in env, config, SSO, role
+
+Options
+-------
+--help, -h     this
+--ec2,         get credentials from server IAM profile
+--env          get credentials from environment variables
+--profile, -p  get credentials from profile in credentials configuration
+--role         get credentials from SSO role
+--account      use with --role, specify AWS account id
+
+ $ amazon-credentials.sh --profile=test
+  export AWS_ACCESS_KEY_ID=AKI*****************
+  export AWS_SECRET_ACCESS_KEY=****************************************
+
+ $ aws sso login
+ $ amazon-credentials.sh --role my-sso-role --account 01234567890
+EOT
+
+  exit 0;
+}
+
+########################################################################
+sub main {
+########################################################################
+  my ($self) = @_;
+
+  use Getopt::Long;
+
+  my %options;
+
+  if (
+    !GetOptions(
+      \%options, 'help', 'env', 'ec2', 'profile=s', 'role=s', 'account=s'
+    )
+  ) {
+    help();
+  }
+
+  if ( $options{help} ) {
+    help();
+  }
+
+  if ( $options{role} && $options{account} ) {
+    my $credentials
+      = Amazon::Credentials::set_sso_credentials(
+      @options{qw{role account region}} );
+
+    foreach
+      my $key (qw{AWS_ACCESS_KEY_ID AWS_SECRET_ACCESS_KEY AWS_SESSION_TOKEN})
+    {
+      printf {*STDOUT} sprintf "export %s=%s\n", $key, $ENV{$key} // EMPTY;
+    }
+
+    exit 0;
+  }
+
+  my @order;
+
+  if ( $options{env} ) {
+    push @order, 'env';
+  }
+  elsif ( $options{ec2} ) {
+    push @order, 'ec2';
+  }
+  elsif ( $options{profile} ) {
+    push @order, 'file';
+  }
+
+  my $creds = $self->new(
+    profile => $options{profile} // EMPTY,
+    order   => ( @order ? \@order : DEFAULT_SEARCH_ORDER )
+  );
+
+  print {*STDOUT} $creds->format_credentials("export %s=%s\n");
+
+  exit 0;
+}
+
 1;
 
 __END__
@@ -1249,6 +1481,10 @@ Amazon::Credentials - fetch Amazon credentials from file, environment or role
  my @order = qw{ env file container role };
  my $creds = Amazon::Credentials->new( { order => \@order } );
 
+CLI
+
+ amazon-credentials.sh --help
+
 =head1 DESCRIPTION
 
 Class to find AWS credentials from either the environment,
@@ -1259,9 +1495,31 @@ to determine the order in which the class will look for credentials.
 The default order is I<environent>, I<file>, I<container>, I<instance
 meta-data>. See L</new>.
 
+I<NEW!>
+
+This class also implements a method for retrieving your SSO
+credentials. By default the method will set the environment variables
+C<AWS_ACCESS_KEY_ID>, C<AWS_SECRET_ACCESS_KEY> and
+C<AWS_SESSION_TOKEN>. Subsequently call C<Amazon::Credentials> to
+retrieve and use the credentials from the localized environment. If
+you only want to retrieve the credentials use C<get_role_credentials>.
+
+ use Amazon::Credentials qw{set_sso_credentials get_role_credentials};
+
+ set_sso_credentials($role_name, $account_id, $region);
+ my $credentials = Amazon::Credentials->new;
+
+ my $credential = get_role_credentials(role_name  => $role_name,
+                                       account_id => $account_id,
+                                       region     => $region);
+
+or from the command line...
+
+ amazon-credentials.sh --role my-sso-role --account 01234567890
+
 =head1 VERSION
 
-This document reverse to verion 1.1.9 of
+This document reverse to verion 1.1.10 of
 C<Amazon::Credentials>.
 
 =head1 METHODS AND SUBROUTINES
@@ -1647,6 +1905,48 @@ I<Note that you should never have to call
 this method. If you call this method it will ignore your cache
 setting!>
 
+=head1 SSO CREDENTIALS
+
+You can retrieve your SSO credentials after logging in using the
+C<sso_set_credentials> or C<get_role_credentials> methods.
+
+After logging in using your SSO credentials...
+
+ aws sso login
+
+...call one of the methods below to retrieve your credentials.
+
+=head2 get_role_credentials
+
+ get_role_credentials( options )
+
+C<options> is a hash (not reference) of options
+
+=over 5
+
+=item role_name => role name (required)
+
+=item account_id => AWS account id (required)
+
+required
+
+=ite region => AWS region 
+
+default: $ENV{AWS_REGION}, $ENV{AWS_DEFAULT_REGION}, us-east-1
+
+=head2 set_sso_credentials
+
+ set_sso_options(role-name, account-id, region)
+
+Calls C<get_role_credentials> and set AWS credenital environment
+variables. Region is optional, all other parameters are required.
+
+ use Amazon::Credentials qw{set_sso_credentials}
+
+ set_sso_credentials(@ENV{qw{AWS_ROLE_NAME AWS_ACCOUNT_ID}});
+
+ my $credentials = Amazon::Credentials->new;
+
 =head1 SETTERS/GETTERS
 
 All of the options described in the new method can be accessed by a
@@ -1819,18 +2119,18 @@ can and should encrypt credentials in a more secure way.
 By default the module will generate its own random passkey during
 initialization and use that to encrypt and decrypt the
 credentials. Obviously the passkey must be available for
-C<Amazon::Credentials> to decrypt the keys and thus it is stored with
-the credentials which is less than ideal. To avoid storing the passkey
-with the credentials, pass a reference to a subroutine that will
-provide the passkey for encryption and decryption. You can even use
-the same passkey generator that is used by C<Amazon::Credentials>
-(C<create_passkey>).
+C<Amazon::Credentials> to decrypt the keys, however it is B<NOT>
+stored in the blessed hash reference that stores other data used by
+the class. To avoid having the class know about your passkey at all, pass a
+reference to a subroutine that will provide the passkey for encryption
+and decryption. You can even use the same passkey generator that is
+used by C<Amazon::Credentials> (C<create_passkey>).
 
-The point here is to avoid storing credentials in the same object as
-the credentials to minimize the likelihood of exposing your
-credentials or your methods for encryption in logs...better but
-not perfect. It's still possible to expose your passkey and your
-credentials if you are not careful.
+The point here is to avoid storing your passkey the same object as the
+credentials to minimize the likelihood of exposing your credentials or
+your methods for encryption in logs...better but not perfect. It's
+still may be possible to expose your passkey and your credentials if
+you are not careful.
 
  use Amazon::Credentials qw{ create_passkey };
 
@@ -1958,6 +2258,14 @@ a method to refresh them when they have.
    $credentials->refresh_token;
  }
 
+=head2 Use Granular Credentials
+
+Consider the APIs that you are calling with these credentials. If all
+you need to do is access a bucket or a key within a bucket, use
+credentials that B<ONLY> allow access to that bucket.  IAM permissions
+can be quite specific regarding what and from where credentials can be
+used to access resources.
+
 =head2 Additonal Notes on Logging
 
 Versions I<1.0.18> and I<1.0.19> allowed you to enable debugging by
